using System;
using System.Text;
using System.Security.Cryptography; // RijndaelManaged
using Microsoft.Practices.EnterpriseLibrary.Common.Cryptography; // SymmetricCryptographer

namespace CUS.ICS.LoginCaptureCrypto
{
	/// <summary>
	/// Borrowed from SymmetricCryptographerBase64 found in Dianne Britton's Login With Proxy portlet made for 
	/// Jenzabar Internet Campus Solution portal and available at http://www.jicswiki.com
	/// </summary>
	public class SymmetricCryptographerBase64
	{
		// SymmetricCryptographer apparently has salt built-in -- Hooray!
		SymmetricCryptographer cryptographer;
		RijndaelManaged mCryptoService = new RijndaelManaged();
		byte[] encryptKey;
		ASCIIEncoding encoder = new ASCIIEncoding();

		public SymmetricCryptographerBase64(string key)
		{
			this.mCryptoService.Mode = CipherMode.CBC;;
			this.encryptKey = GetLegalKey(key);
			this.cryptographer = new SymmetricCryptographer(mCryptoService,encryptKey);
		}

		public string Encrypt(string plaintext)
		{
			byte[] encryptedPlaintext = cryptographer.Encrypt(encoder.GetBytes(plaintext));
			return System.Convert.ToBase64String(encryptedPlaintext);
		}

		public string Decrypt(string plaintext)
		{
			try
			{
				return encoder.GetString(cryptographer.Decrypt(System.Convert.FromBase64String(plaintext)));
			}
			catch
			{
				return "baddecrypt";
			}
		}

		// original author: Chidi C. Ezeukwu
		// http://www.codeproject.com/KB/security/chidi_cryptommetric.aspx
		// very minorly modified by Dianne E. Britton
		protected virtual byte[] GetLegalKey(string mKey)
		{
			// Adjust key if necessary, and return a valid key
			if (mCryptoService.LegalKeySizes.Length > 0)
			{
				// Key sizes in bits
				int keySize = mKey.Length * 8;
				int minSize = mCryptoService.LegalKeySizes[0].MinSize;
				int maxSize = mCryptoService.LegalKeySizes[0].MaxSize;
				int skipSize = mCryptoService.LegalKeySizes[0].SkipSize;

				if (keySize > maxSize)
				{
					// Extract maximum size allowed
					mKey = mKey.Substring(0, maxSize / 8);
				}
				else if (keySize < maxSize)
				{
					// Set valid size
					int validSize = (keySize <= minSize)? minSize : 
						(keySize - keySize % skipSize) + skipSize;
					if (keySize < validSize)
					{
						// Pad the key with asterisk to make up the size
						mKey = mKey.PadRight(validSize / 8, '*');
					}
				}
			}
			PasswordDeriveBytes key = new PasswordDeriveBytes(mKey, ASCIIEncoding.ASCII.GetBytes(""));
			return key.GetBytes(mKey.Length);
		}
	}     
}
